from django.db import models
from django.utils.translation import gettext_lazy as _


class DiscordIntegration(models.Model):
    name: models.CharField = models.CharField(
        max_length=30,
        unique=True,
        default='default',
        verbose_name=_('logger name'),
        help_text=_('The name to specify as a keyword argument in the Django logging settings.'),
    )
    bot_name: models.CharField = models.CharField(
        max_length=30,
        blank=True,
        verbose_name=_('discord bot username'),
        help_text=_('Override the bot username set in the Discord settings.'),
    )
    avatar_url: models.URLField = models.URLField(
        blank=True,
        verbose_name=_('discord bot avatar'),
        help_text=_('Override the bot avatar set in the Discord settings.'),
    )
    webhook_url: models.URLField = models.URLField(
        verbose_name=_('discord webhook url'),
        help_text=_('The Discord webhook url found in the Discord settings.'),
    )

    def __str__(self) -> str:
        return self.name



# Generated by Django 3.2.6 on 2022-03-13 21:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discord_integration', '0002_auto_20200412_1649'),
    ]

    operations = [
        migrations.AddField(
            model_name='discordintegration',
            name='name',
            field=models.CharField(default='default', help_text='The name to specify as a keyword argument in the Django logging settings.', max_length=30, unique=True, verbose_name='logger name'),
        ),
    ]



from django.contrib import admin

from discord_integration.models import DiscordIntegration


class DiscordIntegrationAdmin(admin.ModelAdmin):
    list_display = ('name',)
    search_fields = ('name',)


admin.site.register(DiscordIntegration, DiscordIntegrationAdmin)



# Generated by Django 2.2.6 on 2020-04-12 20:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discord_integration', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='discordintegration',
            name='avatar_url',
            field=models.URLField(blank=True, help_text='Override the bot avatar set in the Discord settings.', verbose_name='discord bot avatar'),
        ),
        migrations.AlterField(
            model_name='discordintegration',
            name='webhook_url',
            field=models.URLField(help_text='The Discord webhook url found in the Discord settings.', verbose_name='discord webhook url'),
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DiscordIntegrationConfig(AppConfig):
    name = 'discord_integration'
    verbose_name = _('Discord Integration')
    default_auto_field = 'django.db.models.AutoField'



from setuptools import find_packages, setup

with open('README.md') as f:
    readme = f.read()

setup(
    name='django-discord-integration',
    version='1.2.6',
    author='Evan Zhang',
    install_requires=[],
    description='Discord integration for Django, supporting error reporting via webhooks.',
    long_description=readme,
    long_description_content_type='text/markdown',
    url='https://github.com/Ninjaclasher/django-discord-integration',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Operating System :: OS Independent',
        'Topic :: Software Development',
        'Topic :: Communications :: Chat',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.7',
    ],
)



import json
import traceback
from logging import LogRecord
from typing import Any, Dict, Optional, Tuple
from urllib import request

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.log import AdminEmailHandler
from django_stubs_ext import StrOrPromise
from urllib3 import encode_multipart_formdata

__all__ = ['DiscordMessageHandler', 'SimpleDiscordMessageHandler']


MESSAGE_LIMIT: int = getattr(settings, 'DISCORD_MESSAGE_LIMIT', 4000)
BOT_USERNAME: Optional[str] = getattr(settings, 'DISCORD_BOT_USERNAME', None)
AVATAR_URL: Optional[str] = getattr(settings, 'DISCORD_AVATAR_URL', None)

COLORS: Dict[str, int] = getattr(settings, 'DISCORD_COLORS', {
    'ERROR': 0xe74c3c,
    'WARNING': 0xe78c3c,
    'INFO': 0xe7ec3c,
})


class DiscordWebhook:
    def __init__(self, model_name: str) -> None:
        from discord_integration.models import DiscordIntegration

        self.model: Optional[DiscordIntegration]
        try:
            self.model = DiscordIntegration.objects.get(name=model_name)
        except DiscordIntegration.DoesNotExist:
            self.model = None

    def message(self, message: Dict[str, Any], full_message: Optional[str] = None) -> None:
        if self.model is None:
            raise ImproperlyConfigured('Discord integration model does not exist.')

        username = self.model.bot_name or BOT_USERNAME
        if username and 'username' not in message:
            message['username'] = username

        avatar_url = self.model.avatar_url or AVATAR_URL
        if avatar_url and 'avatar_url' not in message:
            message['avatar_url'] = avatar_url

        files = []
        if full_message:
            files.append(('full_message.txt', full_message, 'text/plain'))

        data: Dict[str, Tuple[Optional[str], str, str]] = {
            'payload_json': (None, json.dumps(message), 'application/json'),
        }
        for i, file in enumerate(files):
            data['files[{}]'.format(i)] = file

        # We want to pass in None for the filename, but for some reason the
        # typing here doesn't allow that even though the underlying code
        # supports it. Ignore the type error for now.
        body, header = encode_multipart_formdata(data)  # type: ignore[arg-type]

        req = request.Request(
            self.model.webhook_url,
            data=body,
            headers={'User-Agent': 'Python/3', 'Content-Type': header},
        )
        request.urlopen(req)

    @staticmethod
    def escape(text: str) -> str:
        escape_chars = ('*', '_', '~', '`', '|')
        for ch in escape_chars:
            text = text.replace(ch, '\\' + ch)
        return text

    @staticmethod
    def format_codeblock(text: str, language: str = '') -> str:
        return '```{}\n{}\n```'.format(language, text)


class DiscordMessageHandler(AdminEmailHandler):
    def __init__(self, model_name: str = 'default', **kwargs: Any) -> None:
        super().__init__(**kwargs)
        self.model_name = model_name

    def _get_raw_insecure_uri(self, record: LogRecord) -> str:
        """
        Return an absolute URI from variables available in this request. Skip
        allowed hosts protection, so may return insecure URI.

        From Django source code, views/debug.py
        """
        if not hasattr(record, 'request'):
            return ''
        request = record.request
        return '{scheme}://{host}{path}'.format(
            scheme=request.scheme,
            host=request._get_raw_host(),
            path=request.get_full_path(),
        )

    def _get_traceback(self, record: LogRecord) -> str:
        if record.exc_info:
            return ''.join(traceback.format_exception(*record.exc_info))
        else:
            return record.getMessage()

    def emit(self, record: LogRecord) -> None:
        self.__level = record.levelname
        self.__request_uri = self._get_raw_insecure_uri(record)
        self.__traceback = self._get_traceback(record)
        return super().emit(record)

    def send_mail(self, subject: StrOrPromise, message: StrOrPromise, *args: Any, **kwargs: Any) -> None:
        webhook = DiscordWebhook(self.model_name)

        webhook.message(
            {
                'embeds': [{
                    'title': webhook.escape(str(subject)),
                    'url': self.__request_uri,
                    'description': webhook.format_codeblock(self.__traceback[:MESSAGE_LIMIT], 'py'),
                    'color': COLORS.get(self.__level, 0xeee),
                }],
            },
            full_message=str(message),
        )


class SimpleDiscordMessageHandler(DiscordMessageHandler):
    def send_mail(self, subject: StrOrPromise, message: StrOrPromise, *args: Any, **kwargs: Any) -> None:
        super().send_mail(subject, '', *args, **kwargs)
